/*

        Widget Gallery Feed
        
        Version 1.0
        
        Design & Code: Andy Chentsov (chentsov@gmail.com)
                
        Based on:
        ------------------------------
        G4M RSS-Feed 1.1
        
        ...which is based on:
        German IT-News Viewer
        by Jan Renz <jan@opsm.net>
        
        ...which is based on:
        Tech News Feed
        by Richard Baskett <rick@baskettcase.com>
        
        ..which is based on:
        Check Widgits
        by Lars Blsj (code) and Fredrik Marcus (design)
        ------------------------------
        
        (C) 2005 Extensible Software <http://www.extensiblesoft.com>

*/

// -- Presettings --

cacheFile       = system.widgetDataFolder + "/cache.xml";

feedURL         = "http://www.widgetgallery.com/index.xml";

firstTime       = true;

useCurl         = true; // Use Curl or not - since Konfab@OSX and Konfab@Windows support it, use it
textMaxlen              = 53;           // The maximum length for a text in a row
rowMax          = 10;           // Number of maximum rows in a category
widgetFocused   = true; // Whether or not the widget has the user's focus - true when starting
widgetRunning   = false;        // Logs whether the widget is running now or not

catNew          = 0;
catUpdated      = 1;

categories              = [             // The category names
                        'New', 
                        'Updated'
                        ];
categoryNames           = [             // The category names
                        'New Widgets', 
                        'Updated Widgets'
                        ];
categoryRows    = [];   // Contains the row images of the categories
categoryIcons   = [];   // Contains the icon references of the categories
categoryTexts   = [];   // Contains the text references of the categories
pictureArr              = [];           // Contains all row images that are placed behind the text rows
textObjArr              = [];           // Contains all text objects for the rows
dateObjArr              = [];           // Contains all date objects for the rows
textArr         = [];           // Contains all texts for the rows

updateCounter   = 0;            // Counts the number of minutes since the last update
updateInterval  = 0;            // Number of minutes until the next update (0 = 1 minute, 1 = 2 minutes, etc.)

// Top left corner of the widget without the shadows
baseH                   = 0;            // Basic horizontal position
baseV                   = 0;            // Basic vertical position

baseWidth               = 26;  // Basic width of the widget (without the shadow)
rowHeight               = 18;           // Height of each row

if ("".trim == null) {
  String.prototype.trim = function() {
    return this.replace(/(^\s*)|(\s*$)/g, "");
  }
}

// -- init --
// Initiate the widget by applying some preferences, 
// creating the category rows and creating the rows

function init () {

        if(Number(preferences.prefFirstrun.value)) {
                if(system.platform == "macintosh") {
                        preferences.prefPlatform.value = "Mac";
                } else if(system.platform == "windows") {
                        preferences.prefPlatform.value = "Windows";
                } else {
                        preferences.prefPlatform.value = "Both";
                }
                
                preferences.prefFirstrun.value = 0;
        }
        
        if(preferences.prefPlatform.value == "Mac") {
                feedURL = "http://www.widgetgallery.com/mac_rss.php";
        } else if(preferences.prefPlatform.value == "Windows") {
                feedURL = "http://www.widgetgallery.com/win_rss.php";
        } else {
                feedURL = "http://www.widgetgallery.com/index.xml";
        }
        
        // Apply the preferences
        applyPreferences ();
        
        // Hide the rowPicture
        rowPicture.opacity      = 0;

        // Create Categories
        updateCategories();        
        
        // Reset the text array
        resetTextArr ();
        
        // Create the rows
        updateRows ();

        lastUpdateText.data = "Last update: " + preferences.lastUpdate.value.substr(16,5);
        
        // Display the the main window if it is hidden
        if (!main_window.visible) main_window.visible = true;
}

function updateCategories() {
        // Evaluate the hOffset and vOffset of the first row
        baseH           = headerPicture.hOffset;
        baseV           = headerPicture.vOffset + headerPicture.height;
        // Get the width of the widget (without the shadow)
        baseWidth       = headerPicture.width;
        
        // For each category
        for (var cat = 0; cat < categories.length; cat++) {
                
                // Create the category row image
                categoryRows.push (new Image ());
                categoryRows[cat].src           = "Resources/category.png";
                categoryRows[cat].hOffset       = baseH;
                categoryRows[cat].vOffset       = baseV + (rowHeight * cat);
                categoryRows[cat].width         = baseWidth;
                
                // Create the category icon image
                categoryIcons.push (new Image ());
                categoryIcons[cat].src          = "Resources/category-" + (preferences['expand' + categories[cat]].value == 1 ? "open" : "closed") + ".png";
                categoryIcons[cat].hOffset      = baseH + 2;
                categoryIcons[cat].vOffset      = baseV + (rowHeight * cat) + 2;
                
                // Create the category text
                categoryTexts.push (new Text ());
                categoryTexts[cat].data         = "       " + categoryNames[cat];
                categoryTexts[cat].font         = "Trebuchet MS";
                categoryTexts[cat].size         = 10;
                categoryTexts[cat].hOffset      = baseH + 2;
                categoryTexts[cat].vOffset      = baseV + (rowHeight * cat) + 14;
                //categoryTexts[cat].width      = baseWidth - 4;
                categoryTexts[cat].color        = "#202D45";
                categoryTexts[cat].onMouseEnter = "categoryEnter (" + cat + ")";
                categoryTexts[cat].onMouseExit  = "categoryExit (" + cat + ")";
                categoryTexts[cat].onMouseUp            = "categoryClick (" + cat + ")";

                if (((preferences.displayNew.value == 0) && (cat == 0)) ||
                    ((preferences.displayUpdated.value == 0) && (cat == 1))) {
                  categoryRows[cat].height = 0;
                  categoryRows[cat].opacity = 0;
                  categoryIcons[cat].opacity = 0;
                  categoryTexts[cat].opacity = 0;
                } else {
                  categoryRows[cat].height = 18;
                  categoryRows[cat].opacity = 255;
                  categoryIcons[cat].opacity = 255;
                  categoryTexts[cat].opacity = 255;
                }
        }
}

// -- applyPreferences --
// Apply preferences to category limits, shadows and update interval

function applyPreferences () {
        
        // Get the limits for the categories
        getCategoryLimits ();
        
        // If the shadows are deactivated
        if (preferences.displayShadow.value == 0) {
                // Hide the shadows
                headerShadow.opacity = rowShadow.opacity = footerShadow.opacity = 0;
        }
        // Otherwise
        else {
                // Display the shadows
                headerShadow.opacity = rowShadow.opacity = footerShadow.opacity = 255;
        }

        toolbarPicture.opacity = (preferences.showSearchBar.value == 0)?0:255;
        searchBk.opacity = (preferences.showSearchBar.value == 0)?0:200;
        searchText.opacity = (preferences.showSearchBar.value == 0)?0:255;
        searchBtn.opacity = (preferences.showSearchBar.value == 0)?0:255;
        
        // Evaluate the update interval
        updateInterval     = Number (preferences.prefUpd.value) - 1;
}

// -- getCategoryLimits --
// Update the category limits

function getCategoryLimits () {
        
        // Get the limits for the categories
        limitNew       = preferences.expandNew.value  == 0 ? 0 : Math.min (Number (preferences.limitNew.value), rowMax);
        limitUpdated   = preferences.expandUpdated.value == 0 ? 0 : Math.min (Number (preferences.limitUpdated.value), rowMax);
}

// -- resetTextArr --
// Create an array with an empty first element for each category

function resetTextArr () {
        
        // For each category
        for (var cat = 0; cat < categories.length; cat++) {
                
                // Create an empty array for every category
                textArr[cat]            = [];
                // And an empty first object
                textArr[cat][0]         = new Object ();
                textArr[cat][0].name    = "...";
                textArr[cat][0].link    = "";
        }
}

// -- updateRows --
// Destroy the old and create the new rows and texts
// And adjust the layout

function updateRows () {
        
        // Get the limits for the categories
        getCategoryLimits ();
        
        var totalHeight = 0; // Save the total height of the rows in here
        
        // Clean up
        // For each category
        for (var cat = 0; cat < categories.length; cat++) {
                
                var catName     = categories[cat]; // Get the category's name
                
                // If the picture array contains rows for this category
                // Delete every row picture of this category
                if (pictureArr.length) for (var row in pictureArr[cat]) delete pictureArr[cat][row];
                
                // If the text array contains rows for this category
                // Delete every text object of this category
                if (textObjArr.length) for (var row in textObjArr[cat]) delete textObjArr[cat][row];

                // If the text array contains rows for this category
                // Delete every date object of this category
                if (dateObjArr.length) for (var row in dateObjArr[cat]) delete dateObjArr[cat][row];
        }
        
        // Create rows
        // For each category
        for (var cat = 0; cat < categories.length; cat++) {
                
                var catName     = categories[cat];
                var catRows     = eval ('limit' + catName);
                var catH        = baseH;
                var catV        = categoryRows[cat].vOffset + categoryRows[cat].height;
                
                totalHeight     = catV; // Adjust the total height of the rows
                
                // Create or reset the category in the picture array and in the text array
                pictureArr[cat] = [];
                textObjArr[cat] = [];
                dateObjArr[cat] = [];
                
                // And for each row in this category
                for (var row = 0; row < catRows; row++) {
                        
                        // Create the category row image
                        pictureArr[cat].push (new Image ());
                        pictureArr[cat][row].src        = "Resources/row-" + (row % 2 == 0 ? "odd" : "even") + ".png";
                        pictureArr[cat][row].hOffset    = catH;
                        pictureArr[cat][row].vOffset    = catV + (rowHeight * row);
                        pictureArr[cat][row].width      = baseWidth;
                        pictureArr[cat][row].height     = rowHeight;
                        
                        // Create the category row image
                        textObjArr[cat].push (new Text ());
                        textObjArr[cat][row].data       = "";
                        textObjArr[cat][row].font       = "Tahoma";
                        textObjArr[cat][row].size       = 11;
                        textObjArr[cat][row].hOffset    = catH + 12;
                        textObjArr[cat][row].vOffset    = catV + (rowHeight * row) + 12;
                        textObjArr[cat][row].width      = baseWidth - 30;
                        textObjArr[cat][row].color      = "#333333";

                        // Create the category row image
                        dateObjArr[cat].push (new Text ());
                        dateObjArr[cat][row].alignment  = "right";
                        dateObjArr[cat][row].data       = "";
                        dateObjArr[cat][row].font       = "Tahoma";
                        dateObjArr[cat][row].size       = 11;
                        //dateObjArr[cat][row].hOffset    = baseWidth - 32;
                        dateObjArr[cat][row].hOffset    = baseWidth;
                        dateObjArr[cat][row].vOffset    = catV + (rowHeight * row) + 12;
                        dateObjArr[cat][row].width      = baseWidth - 30;
                        dateObjArr[cat][row].color      = "#333333";
                        
                        // Display the text for this row
                        displayTextRow (cat, row);
                        
                        // Adjust the total height of the rows
                        totalHeight                             += rowHeight
                }
                
                // Adjust following categories
                if (cat < categories.length - 1) {
                        categoryRows[cat + 1].vOffset   = catV + (rowHeight * catRows);
                        categoryIcons[cat + 1].vOffset  = categoryRows[cat + 1].vOffset + 2;
                        categoryTexts[cat + 1].vOffset  = categoryRows[cat + 1].vOffset + 14;
                }
        }

        if (preferences.showSearchBar.value != 0) {
          // Adjust the toolbar
          toolbarPicture.vOffset   = totalHeight;
          searchBk.vOffset         = totalHeight + 2;
          searchText.vOffset       = totalHeight + 2;
          searchBtn.vOffset        = totalHeight + 2;

          totalHeight += 20;
        }
        
        // Adjust the footer
        footerPicture.vOffset   = totalHeight;
        footerText.vOffset      = totalHeight + 11;
        lastUpdateText.vOffset  = totalHeight + 11;
        
        // Adjust the shadows
        rowShadow.height                = totalHeight - headerShadow.height;
        footerShadow.vOffset    = totalHeight;
        
        // Adjust the widget dimensions
        oldHeight = main_window.height;
        main_window.height      = totalHeight + footerShadow.height;

        //if (main_window.vOffset >= Math.round(screen.height / 2)) {
        if ((preferences.resizeUp.value != 0) && (!firstTime)) {
          if (preferences.animateResize.value != 0) {
            main_window.moveTo(main_window.hOffset, main_window.vOffset - (main_window.height - oldHeight), 3);
          } else {
            main_window.vOffset -= (main_window.height - oldHeight);
          }
        }
        firstTime = false;
}

// -- update --
// Initiate the update of the feeds

function update () {
        
        // Display that we are updating
        if (updateText.data == "") updateText.data      = "Updating...";
        
        updateNow ();
        
        // Reset the text array
        resetTextArr ();
        
        // Update the texts
        if (widgetRunning == false) {
                
                // The widget is running by now
                widgetRunning   = true;
                
                // Display the loading messages
                displayAllTextRows ();
        }

        // Load the feeds
        updateInt(feedURL);
}

// -- displayAllTextRows --
// Update every text row within each category

function displayAllTextRows (activateLinks) {
        
        for (var cat = 0; cat < categories.length; cat++) {
                for (var row = 0; row < Math.min (textArr[cat].length, textObjArr[cat].length); row++) {
                        // Display text
                        displayTextRow (cat, row, activateLinks);
                }
        }
}

// -- displayTextRow --
// Display the requested text row
// If a text is available and make it clickable if a link is given

function displayTextRow (cat, row, activateLinks) {
        
        // Set if links shall be activated
        if (activateLinks == undefined) activateLinks = true;
        
        // Empty the text row
        textObjArr[cat][row].data       = "";
        
        // If any name is given
        if (row < textArr[cat].length && textArr[cat][row].name != "") {
                
                var name        = textArr[cat][row].name;
                var updDate     = textArr[cat][row].updDate;
                var desc        = textArr[cat][row].description;
                if (desc) {
                  desc = desc.replace(/\#39;/g, "'");
                }
                
                // If the text is longer than allowed
                // Shorten the text
                if (name.length > textMaxlen) name = name.substring (0, textMaxlen - 1) + " ...";
                
                // Display the name and disable the link
                textObjArr[cat][row].data               = replaceMe (name);
                textObjArr[cat][row].color              = textArr[cat][row].color == undefined ? "#333333" : textArr[cat][row].color;
                textObjArr[cat][row].onMouseEnter       = "";
                textObjArr[cat][row].onMouseExit        = "";
                textObjArr[cat][row].onMouseUp          = "";
                if ((desc != "") && (desc != null)) {
                        textObjArr[cat][row].tooltip            = desc;
                }

                // Display the name and disable the link
                dateObjArr[cat][row].data               = replaceMe (updDate);
                dateObjArr[cat][row].color              = textArr[cat][row].color == undefined ? "#333333" : textArr[cat][row].color;
                
                // Enable the link if given
                if (textArr[cat][row].link != "" && activateLinks == true) {
                        textObjArr[cat][row].onMouseEnter       = "itemEnter (" + cat + ", " + row + ")";
                        textObjArr[cat][row].onMouseExit        = "itemExit (" + cat + ", " + row + ")";
                        textObjArr[cat][row].onMouseUp  = "itemClick (" + cat + ", " + row + ")";
                }
        }
}

// -- updateInt --
// Update this feed

function updateInt (newsURL) {
        
        // Fetch the data from the given URL
        //var urlData     = fetchData (newsURL);
        fetchData (newsURL);

}

function updateIntAsync (url) {
        
        //print("Response: " + url.response);
        //print("Result: " + url.result);

        var urlData = url.result;

        // If any data was retrieved
        if (url.response != 0 && urlData.length && urlData.indexOf ("curl: (") == -1) {
            // Save Cache
            filesystem.writeFile(cacheFile, urlData);
            
            preferences.lastUpdate.value = new Date().toString();

            // Extract and store the required data
            distillWidgets (urlData);
        }
        // Otherwise
        else if (filesystem.itemExists(cacheFile)) {
            urlData = filesystem.readFile(cacheFile);

            // Extract and store the required data
            distillWidgets (urlData);
        } else {
                // Report an error for this category in the first row of a resetted category
                textArr[0]    = [];
                textArr[0][0] = new Object ();
                textArr[0][0].name    = "RSS feeds couldn't be loaded!";
                textArr[0][0].link    = "";
                textArr[0][1] = new Object ();
                textArr[0][1].name    = "Please check your intarweb connection";
                textArr[0][1].link    = "";
                textArr[0][1].color   = "#999999";
                textArr[0][2] = new Object ();
                textArr[0][2].name    = "and try again later.";
                textArr[0][2].link    = "";
                textArr[0][2].color   = "#999999";
        }

        // Update the texts
        displayAllTextRows ();
        
        // Display that we are finished updating
        updateText.data = "";

        lastUpdateText.data = "Last update: " + preferences.lastUpdate.value.substr(16,5);
}

// -- fetchData --
// Fetch the data via Curl or JavaScript method

function fetchData (theURL) {
        /*if (useCurl == true) return runCommand ("curl -s -S '" + theURL + "'");
        else return (new URL()).fetch (theURL);*/

        var u = new URL();
        u.location = theURL;
        u.fetchAsync(updateIntAsync);
}

function dateToDdMmYy(dateString) {
    var date = new Date(dateString); 
    var d  = date.getDate();
    var day = (d < 10) ? '0' + d : d;
    var m = date.getMonth() + 1;
    var month = (m < 10) ? '0' + m : m;
    var y = date.getYear();
    var year = String(y).substr(1);
    return day + "/" + month + "/" + year;
}

function dateToMonthDay(dateString) {
    var date = new Date(dateString); 
    var d  = date.getDate();
    var m = date.getMonth();
    var month = ["Jan", "Feb", "Mars", "April", "May", "June", "July", "Aug", "Sep", "Oct", "Nov", "Dec"][m];
    var y = date.getYear();
    var year = String(y).substr(1);
    var retVal = month + " " + d;
    
    // Check for strange update date "December 31, 1969". date.getYear() gives 103 for 2003 and 69 for 1969
    if(y < 100) retVal = "";
    
    return retVal;
}

function stripTags(str) {
        str = str.replace(/<br\/>/g, ' ');
        str = str.replace(/\&quot\;/g, '"');
        str = str.replace(/\&amp\;/g, '&');
        var retVal = "";
        var i = str.indexOf("<");
        if(i > -1) {
                var bail = 0;
                var e;
                while(bail++ < 10000 && i > -1) {
                        retVal += str.substring(0, i);
                        e = str.indexOf(">", i);
                        str = str.substring(e+1);
                        i = str.indexOf("<"); 
                }
                retVal += str;
        } else {
                retVal = str;
        }
        return retVal;
}

// -- distillWidgets --
// Extract the required data

function distillWidgets (xml) {
        
        var itemTag             = '<item>';
        var itemTag2    = '<item ';
        var itemTag3    = '>';
        var itemEndTag  = '</item>';    
        var linkTag             = '<link>';
        var linkEndTag  = '</link>';
        var titleTag    = '<title>';
        var titleEndTag = '</title>';
        var descriptionTag = '<description><![CDATA[';
        var descriptionEndTag = ']]></description>';
        var dateTag = '<pubDate>';
        var dateEndTag = '</pubDate>';
        var item                = '';
        
        var bailOut             = 0;
        
        var itemIndex   = xml.indexOf (itemTag);
        var itemIndex2  = xml.indexOf (itemTag2);
        
        textArr[catNew]        = [];
        textArr[catUpdated]    = [];

        cat = 0;
        
        while ((itemIndex > -1 || itemIndex2 > -1) && bailOut++ < 100) {
                
                if (itemIndex > -1) {
                        startIndex      = itemIndex + itemTag.length;
                }
                else {
                        startIndex      = xml.indexOf (itemTag3, itemIndex2);
                }
                
                item            = xml.substring (startIndex, xml.indexOf (itemEndTag, startIndex));
                
                theWidget       = new Object ();
                
                // Name
                var itemStartIndex= item.indexOf (titleTag) + titleTag.length;
                theWidget.name  = item.substring (itemStartIndex, item.indexOf (titleEndTag, itemStartIndex));
                
                theWidget.name  = replaceMe(theWidget.name);
                
                // Link
                var itemStartIndex= item.indexOf (linkTag) + linkTag.length;
                theWidget.link  = item.substring (itemStartIndex, item.indexOf (linkEndTag, itemStartIndex));
                theWidget.link  = replaceMe (theWidget.link);
                //log(theWidget.link);

                // "NEW: " or "UPDATED: "
                if(theWidget.name.indexOf("UPDATED: ") == 0) {
                    cat = 1;
                    theWidget.name = theWidget.name.substr("UPDATED: ".length);
                } else {
                    // "NEW: "
                    cat = 0;
                    theWidget.name = theWidget.name.substr("NEW: ".length);
                }

                // Update date 
                itemStartIndex = item.indexOf(dateTag) + dateTag.length;
                theWidget.updDate = dateToMonthDay(item.substring(itemStartIndex, item.indexOf(dateEndTag)));

                // Description
                itemStartIndex = item.indexOf(descriptionTag) + descriptionTag.length;
                theWidget.description = stripTags(item.substring(itemStartIndex, item.indexOf(descriptionEndTag)));
                
                textArr[cat].push (theWidget);
                
                // Next
                itemIndex       = xml.indexOf (itemTag, startIndex + item.length);
                itemIndex2      = xml.indexOf (itemTag2, startIndex + item.length);
        }
}

// -- replaceMe --
// Replace special characters

function replaceMe (myString) {
        
        if (myString == undefined) return "";
        
        var pattern     = /\Wamp;/ig;
        var newString   = myString.replace (pattern,'&');
        
        var pattern     = /<!\[CDATA\[/ig;
        var newString   = newString.replace (pattern,'');
        
        var pattern     = /]]>/ig;
        var newString   = newString.replace (pattern,'');
        
        var pattern     = /\Wquot;/ig;
        var newString   = newString.replace (pattern,"\"");
        
        var pattern     = /\Wapos;/ig;
        var newString   = newString.replace (pattern,"'");
        
        var pattern     = "\\";
        var newString   = newString.replace (pattern,"");
        
        return newString;
}

// -- umlautFix --
// Replace very special umlaut characters

function umlautFix (newString) {
        
        // Jan Renz (jan@opsm.net)
        
        newString = newString.replace ('??',"");
        
        newString = newString.replace ("?","");
        newString = newString.replace ("?","");
        newString = newString.replace ("?","");
        
        newString = newString.replace ("","");
        newString = newString.replace ("","");
        newString = newString.replace ("","");        
        
        newString = newString.replace ("&#39;", "'");
        newString = newString.replace ("&#33;", "!");
        
        return newString;
}

// -- onPreferencesChanged --
// Fired when the preferences are changed - could it be different?

function onPreferencesChanged () {
        
        // Get the preferences
        applyPreferences ();
        
        // Update Categories
        updateCategories();

        // Update the rows
        updateRows ();
}

// -- categoryEnter --
// Fired when the cursor enters a category link

function categoryEnter (cat) {
        
        // Highlight the text of this category
        categoryTexts[cat].color = '#34498B';
}

// -- categoryExit --
// Fired when the cursor exits a category link

function categoryExit (cat) {
        
        // De-highlight the text of this category
        categoryTexts[cat].color = '#202D45';
}

// -- categoryClick --
// Fired when the cursor clicks a category link

function categoryClick (cat) {
        
        prefName        = 'expand' + categories[cat];
        prefRef = preferences[prefName].value;
        
        // Switch the display preferences for this category
        preferences[prefName].value     = prefRef = 1 - prefRef;
        
        categoryIcons[cat].src  = "Resources/category-" + (prefRef == 1 ? "open" : "closed") + ".png";
        categoryIcons[cat].reload ();
        
        // Update the rows
        updateRows ();
}

// -- itemEnter --
// Fired when the cursor enters an item link

function itemEnter (cat, row) {
        
        // Highlight the text of this category
        textObjArr[cat][row].color      = '#000000';
}

// -- itemExit --
// Fired when the cursor exits an item link

function itemExit (cat, row) {
        
        // De-highlight the text of this category
        textObjArr[cat][row].color      = '#333333';
}

// -- categoryEnter --
// Fired when the cursor clicks an item link

function itemClick (cat, row) {
        openURL (textArr[cat][row].link);
}

// -- onGainFocus --
// Fired when the widget gains the user's focus
        
function onGainFocus () {
        
        // Register the focus
        widgetFocused   = true;
        
        // Enable the links if focusClicks are required
        if (preferences.focusClicks.value == 1) {
                
                displayAllTextRows (true);
                
                // If the shadows are activated
                if (preferences.displayShadow.value == 1) {
                        // Fully display the shadows
                        headerShadow.src        = "Resources/header-shadow.png";
                        headerShadow.reload ();
                        rowShadow.src   = "Resources/row-shadow.png";
                        rowShadow.reload ();
                        footerShadow.src        = "Resources/footer-shadow.png";
                        footerShadow.reload ();
                }
        }
}

// -- onGainFocus --
// Fired when the widget loses the user's focus
        
function onLoseFocus () {
        
        // Register the lost focus
        widgetFocused   = false;
        
        // Disable the links if focusClicks are required
        if (preferences.focusClicks.value == 1) {
                
                displayAllTextRows (false);
                
                // If the shadows are activated
                if (preferences.displayShadow.value == 1) {
                        // Fade the shadows
                        headerShadow.src        = "Resources/header-shadow-inactive.png";
                        headerShadow.reload ();
                        rowShadow.src   = "Resources/row-shadow-inactive.png";
                        rowShadow.reload ();
                        footerShadow.src        = "Resources/footer-shadow-inactive.png";
                        footerShadow.reload ();
                }
        }
}

// -- Search Widget --

function searchWidget() {
        if (searchText.data.trim() == "") {
          alert("Please enter widget name.");
          return;
        }
        openURL("http://www.widgetgallery.com/index.php?search=" + searchText.data);
        searchText.data = "";
}

// -- Finally start it all! --

init ();
update ();
